#!/usr/bin/perl -w
##############################################################################
# Prepare data about the given application to be inserted into the database.
#
# This script takes a list of components to be processed from file and
# calls 'componenttodb.pl' for each file. The results are collected in
# one file that can be used by 'upload_app_data.pl' for adding information
# to the database.
#
# Author: Denis Silakov <silakov@ispras.ru>
##############################################################################

use strict;
use Getopt::Long;

use Text::Wrap;
Getopt::Long::Configure ("posix_default", "no_ignore_case");

use Distros;
# Application information
my $AppName="";
my $AppVer="";
my $ComponentsFile;
my $AppFilesList;
my $Comment='';
my $Aarch="";
my $OutputFile;
my $Packager="";
my $URL="";

# Package manager used in the distribution
my $PackageManager;

sub HELP_MESSAGE();
sub guess_platform($ );
sub get_app_version();
sub get_app_comment();
sub get_app_packager();
sub get_app_url();

# options
my $opt_help;
my $quiet;

GetOptions("help|h!" => \$opt_help,
            "comment|c=s" => \$Comment,
            "outfile|o=s" => \$OutputFile,
            "arch|a=s" => \$Aarch,
            "version|v=s" => \$AppVer,
            "infile|i=s" => \$ComponentsFile,
            "filelist|l=s" => \$AppFilesList,
            "packagemanager|m=s" => \$PackageManager,
            "packager|p=s" => \$Packager,
            "url|u=s" => \$URL,
            "quiet|q!" => \$quiet);

HELP_MESSAGE() if (defined $opt_help);

if(@ARGV != 1) {HELP_MESSAGE(); exit(1);}

# Guess package manager, if it was not specified
if( (not $PackageManager) and (not $AppFilesList) ) {
    $PackageManager = guess_package_manager();
    if( $PackageManager eq "unknown" ) {
        print STDERR "Cannot guess package manager! Please specify it directly using '-m' option.";
        exit(1);
    }
    else {
        print STDERR "No package manager was specified, guessing it is $PackageManager\n";
    }
}

$AppName = $ARGV[0];

if( !$Packager ) {
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify packager manually when you use files or components list.\n";
	}
	else {
		get_app_packager();
	}
}

if( !$URL ) {
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify packager manually when you use files or components list.\n";
	}
	else {
		get_app_url();
	}
}

if( !$AppVer ) {
    # Guess package manager, if it was not specified
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify version manually when you use files or components list.\n";
    }
    else {
        get_app_version();
    }
}

if( not $Aarch ) {
    $Aarch = guess_platform(`uname -m`);
}

if( not $OutputFile ) {
    $OutputFile = $AppName."_".$AppVer."_".$Aarch."_upload_data";
}

##############################################################################
# Main section
##############################################################################

if( !$Comment ) {
    # Guess package manager, if it was not specified
    if( $AppFilesList ) {
        print STDERR "If you want to add summary for you application, specify it with '-c' option.\n";
    }
    else {
        get_app_comment();
    }
}

system "echo \"!Application '$AppName' '$AppVer' '$Aarch' '$Packager' '$URL' '$Comment'\" > \"$OutputFile\"";

my $options = "";
if( $quiet ) {
    $options.= " -q ";
}
if( $AppFilesList ) {
    $options.= " -l $AppFilesList ";
}
if( $PackageManager ) {
    $options.= " -m $PackageManager ";
}

if( $ComponentsFile ) {
    open( COMPONENT_LIST, "<$ComponentsFile" );

    while( <COMPONENT_LIST> )
    {
        chomp;
        # in the input file we can find either the component name
        # and the package that provide it or only the component name
        if( /((\w|\:|\-|\.|\d)+) (.+)/) {
            system "./componenttodb.pl $options -c $1 $3";
            system "cat \"".$3."_upload_data\" >> \"$OutputFile\"";
            system "rm \"".$3."_upload_data\"";
        }
        else {
            system "./componenttodb.pl $options $_";
            system "cat \"".$_."_upload_data\" >> \"$OutputFile\"";
            system "rm \"".$_."_upload_data\"";
        }
    }
}
else {
    # No file with packages - let's take application name as the only package
    system "./componenttodb.pl $options \"$AppName\"";
    system "cat \"".$AppName."_upload_data\" >> \"$OutputFile\"";
    system "rm \"".$AppName."_upload_data\"";
}

##############################################################################
# Miscellaneous functions - obtain different application information
##############################################################################

sub get_app_packager()
{
	my $PkgLine;

    if( $PackageManager eq "rpm" ) {
    	# 'Distribution' field (if any) is usually the most informative
		$PkgLine = `rpm -q --info $AppName | grep Distribution | cut -d\\: -f2- `;

		if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
			$PkgLine = `rpm -q --info $AppName | grep Vendor | cut -d\\: -f3- `;
		}

		# 'Packager' field can contain email embreced with '<>' - cut it
		if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
			$PkgLine = `rpm -q --info $AppName | grep Packager | cut -d\\: -f2- | cut -d\\< -f1`;
		}

		$Packager = trim($PkgLine);
		if( !$Packager ) {
            print STDERR "Can't detect application packager!\n";
		}
    }
    elsif( $PackageManager eq "dpkg" ) {
    	# Actually, two 'Maintainer' fields can exist - 'Maintainer' and 'Original-Maintainer'
    	# The former will be printed first and this is what we need. Otherwise 'Original-Maintainer'
    	# is our only choise.
		$PkgLine = `dpkg -s $AppName | grep Maintainer | cut -d\\: -f2- | cut -d\\< -f1`;
		$Packager = trim($PkgLine);
		if( !$Packager ) {
            print STDERR "Can't detect application packager!\n";
		}
    }
    elsif( $PackageManager eq "emerge" ) {
    	$Packager = 'Gentoo';
    }
    else {
        print STDERR "Can't detect application packager - package manager '$PackageManager' is not supproted\n";
    }
}

sub get_app_url()
{
	my $URLLine;

    if( $PackageManager eq "rpm" ) {
		$URLLine = `rpm -q --info $AppName | grep URL | cut -d\\: -f2- `;
		$URL = trim($URLLine);
		if( !$URL ) {
            print STDERR "Can't detect application url!\n";
		}
    }
    elsif( $PackageManager eq "dpkg" ) {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted yet\n";
    }
    elsif( $PackageManager eq "emerge" ) {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted yet\n";
    }
    else {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted\n";
    }
}

sub get_app_version()
{
    my $VerLine;

    if( $PackageManager eq "rpm" ) {
        $VerLine = `rpm -qi $AppName | grep Version | grep Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s+\:\s+((\w|\.|\-|\_|\d)+)\s+Vendor/ ) {
            $AppVer = $1;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $VerLine = `dpkg -s $AppName | grep -i version`;
        chomp($VerLine);
		if( $VerLine =~ /(V|v)ersion\s*:\s*(\S+)/ ) {
            $AppVer = $2;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $VerLine = `emerge --pretend --nodeps $AppName | grep $AppName`;
         chomp($VerLine);
        if( $VerLine =~ /\[(\s|\w|\d)*\](\s|\w|\/|\d|\.|\-|\+)*\[((\s|\w|\d|\.|\-|\+)*)\]/ ) {
            $AppVer = $3;
        }
        elsif( $VerLine =~ /no\ ebuilds/ ) {
            print STDERR "Can't detect application version - it seems that it is not installed\n";
        }
        elsif( $VerLine =~ /$AppName(\s|\-)*((\w|\.|\d|\-|\+)+)/ ) {
            $AppVer = $2;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    else {
        print STDERR "Can't detect application version - package manager '$PackageManager' is not supproted\n";
    }
}

sub get_app_comment()
{
    my $CommentLine;

    if( $PackageManager eq "rpm" ) {
        $CommentLine = `rpm -qi $AppName | grep Summary`;
        if( $CommentLine =~ /Summary\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            print STDERR "Can't get application summary!\n";
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $CommentLine = `dpkg -s $AppName | grep Description`;
        if( $CommentLine =~ /Description\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            print STDERR "Can't get application summary!\n";
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        if( does_shell_know("equery") ) {
            print STDERR "Can't get application summary - obtaining summary from emerge is not supported yet\n";
        }
        else {
            print STDERR "Can't get application summary - 'equery' tool from 'gentoolkit' package is required.\n";
        }
    }
    else {
        print STDERR "Can't get application summary - package manager '$PackageManager' is not supproted\n";
    }
}

sub HELP_MESSAGE()
{
print STDERR <<"EOM"

$0: Obtain information about application.

Usage: $0 [options] app_name

app_name                      Name of the application

-a|--arch                     Specify architecture. Possible values are IA32, IA64, PPC32,
                              PPC64, S390, S390X and x86-64. By default the output of `uname -m`
                              is analyzied
-h|--help                     Display this help message
-v|--version app_version      Application version. If none is specified, the script will try to obtain it from package manager
-i|--infile filename          File with the list of packages to be processed (i.e. packages that form the application)
                              If no file is specified, application name is considered to be the only package
-l|--filelist filename        File containing list of file names that are installed by the application.
                              (Directory names are also allowed; all files from given directories will be processed)
-o|--outfile filename         Filename where the data for upload will be situated
                              By default, "<app_name>_<app_ver>_<arch>_upload_data" is used
-q|--quiet                    Do not print any information messages
-u|--url URL                  Application homepage URL
-p|--packager packager        Application packager - any information about uploaded application build
                              (distribution, maintainer, etc.). This can be useful since different builds of the same
                              application may require different sets of external interfaces. If nothing is specified,
                              then the script will try to guess it using package manager
-m|--packagemanager pkg_name  Package manager used in the distribution. Supported values
                              are 'rpm', 'dpkg' and 'emerge'. If none is specified, the script
                              will try to guess it
-c|--comment comment_text     Comment for the application (will be inserted into the database). If none is specified, the
                              script will try to obtain it from package manager

EOM
      ; exit(1);
}
